/*
 * yasapi_config_list.c
 * Copyright (C) 2015 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>
#include <resources.h>

///////////////////////////////////////////////////////////////////////////////
#if defined (YASAPI_DEBUG) // {
static const ControlComboBoxList gcaDebugList[]={
  { L"Off",0 },
  { L"Default",1 },
  { L"Verbose",2 },
  { NULL,0 }
};

static const ControlComboBoxConfig gcDebugConfig=
  { IDC_COMBOBOX_DEBUG,gcaDebugList };

////////////////
static const ControlComboBoxList gcaSleepList[]={
  { L"0 sec",0 },
  { L"2 sec",2*1000 },
  { L"5 sec",5*1000 },
  { L"10 sec",10*1000 },
  { L"20 sec",20*1000 },
  { L"1 min",1*60*1000 },
  { L"2 min",2*60*1000 },
  { L"5 min",5*60*1000 },
  { NULL,0 }
};

static const ControlComboBoxConfig gcSleepConfig=
  { IDC_COMBOBOX_SLEEP,gcaSleepList };
#endif // }

////////////////
static const ControlCheckBoxConfig gcMono2StereoConfig=
  { IDC_CHECKBOX_MONO2STEREO,0,1 };

////////////////
static const ControlCheckBoxConfig gcVolumeConfig=
  { IDC_CHECKBOX_VOLUME,0,1 };

////////////////
static const ControlComboBoxList gcaAudioClockList[]={
  { L"WASAPI",1 },
  { L"System",0 },
  { NULL,0 }
};

static const ControlComboBoxConfig gcAudioClockConfig=
  { IDC_COMBOBOX_CLOCK,gcaAudioClockList };

////////////////
const Control gcaPluginControlList[]={
#if defined (YASAPI_DEBUG) // {
  { &gcComboBoxType,offsetof(PluginOptions,debug),&gcDebugConfig },
  { &gcComboBoxType,offsetof(PluginOptions,sleep),&gcSleepConfig },
#endif // }
  { &gcCheckBoxType,offsetof(PluginOptions,mono2stereo),&gcMono2StereoConfig },
  { &gcCheckBoxType,offsetof(PluginOptions,volume),&gcVolumeConfig },
  { &gcComboBoxType,offsetof(PluginOptions,audioclock),&gcAudioClockConfig },
  { NULL,0,NULL }
};

///////////////////////////////////////////////////////////////////////////////
static const ControlComboBoxList gcaShareModeList[]={
  { L"Shared",YASAPI_SHAREMODE_SHARE },
  { L"Exclusive",YASAPI_SHAREMODE_EXCLUSIVE },
  { L"Automatic",YASAPI_SHAREMODE_AUTOMATIC },
  { NULL,0 }
};

static const ControlComboBoxConfig gcShareModeConfig=
  { IDC_COMBOBOX_SHAREMODE,gcaShareModeList };

////////////////
static const ControlCheckBoxConfig gcAutoConvertPCMConfig=
  { IDC_CHECKBOX_AUTOCONVERT_PCM,FALSE,TRUE };

////////////////
static const ControlCheckBoxConfig gcSRCDefaultQualityConfig=
  { IDC_CHECKBOX_SRC_DEFAULT_QUALITY,FALSE,TRUE };

////////////////
static const ControlComboBoxList gcaPullList[]={
  { L"Push",FALSE },
  { L"Pull",TRUE },
  { NULL,0 }
};

static const ControlComboBoxConfig gcPullConfig=
  { IDC_CHECKBOX_PULL,gcaPullList };

////////////////
const Control gcaDeviceControlList[]={
  { &gcComboBoxType,offsetof(ChannelOptions,eShareMode),
      &gcShareModeConfig },
  { &gcCheckBoxType,offsetof(ChannelOptions,bAutoConvertPCM),
      &gcAutoConvertPCMConfig },
  { &gcCheckBoxType,offsetof(ChannelOptions,bSRCDefaultQuality),
      &gcSRCDefaultQualityConfig },
  { &gcComboBoxType,offsetof(ChannelOptions,bPull),
      &gcPullConfig },
  { NULL,0,NULL }
};

///////////////////////////////////////////////////////////////////////////////
static const ControlCustSliderConfig gcRingSizeConfig=
  { IDC_SLIDER_RING_SIZE,-1,IDC_STATIC_RING_SIZE };

////////////////
static const ControlCustSliderConfig gcRingFillConfig=
  { IDC_SLIDER_RING_FILL,IDC_SLIDER_RING_SIZE,IDC_STATIC_RING_FILL };

////////////////
static const ControlCustSliderConfig gcShareSizeConfig=
  { IDC_SLIDER_SHARE_SIZE,IDC_SLIDER_RING_FILL,IDC_STATIC_SHARE_SIZE };

////////////////
const Control gcaBuffersControlList[]={
  { &gcCustSliderType,offsetof(ChannelOptions,qRingSize),&gcRingSizeConfig },
  { &gcCustSliderType,offsetof(ChannelOptions,qRingFill),&gcRingFillConfig },
  { &gcCustSliderType,offsetof(ChannelOptions,qShareSize),&gcShareSizeConfig },
  { NULL,0,NULL }
};
