/*
 * yasapi_config_slider.c
 * Copyright (C) 2015 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>
#include <resources.h>

///////////////////////////////////////////////////////////////////////////////
static void CustSliderGetRect(HWND hDlg, int nId, RECT *rc)
{
  GetWindowRect(
    GetDlgItem(hDlg,nId),       // _In_   HWND hWnd,
    rc                          // _Out_  LPRECT lpRect
  );

  ScreenToClient(
    hDlg,                       // _In_  HWND hWnd,
    (LPPOINT)&rc->left          // LPPOINT lpPoint
  );

  ScreenToClient(
    hDlg,                       // _In_  HWND hWnd,
    (LPPOINT)&rc->right         // LPPOINT lpPoint
  );
}

static double CustSliderGet(HWND hDlg, int nIdSlider)
{
  double pos;

  pos=SendMessage(
    GetDlgItem(hDlg,nIdSlider),
                                // _In_  HWND hWnd,
    TBM_GETPOS,                 // _In_  UINT Msg,
    0,                          // _In_  WPARAM wParam,
    0                           // _In_  LPARAM lParam
  );

  return pos/CUST_SLIDER_SCALE;
}

void CustSliderSetWidth(HWND hDlg, int nIdSlider, int nIdParent)
{
  enum { MIN_WIDTH=27 };
  RECT rcSlider,rcParent;
  int pos,max,width;
  double q;

  // get the parent's maximum range.
  max=SendMessage(
    GetDlgItem(hDlg,nIdParent),
                                // _In_  HWND hWnd,
    TBM_GETRANGEMAX,            // _In_  UINT Msg,
    0,                          // _In_  WPARAM wParam,
    0                           // _In_  LPARAM lParam
  );

  // get the parent's position.
  pos=SendMessage(
    GetDlgItem(hDlg,nIdParent),
                                // _In_  HWND hWnd,
    TBM_GETPOS,                 // _In_  UINT Msg,
    0,                          // _In_  WPARAM wParam,
    0                           // _In_  LPARAM lParam
  );

  // calculate the scaling factor.
  q=pos-CUST_SLIDER_MIN;

  if (0.0<q)
    q/=(max-CUST_SLIDER_MIN);

  // calculate the slider's width based on the parent's width.
  CustSliderGetRect(hDlg,nIdSlider,&rcSlider);
  CustSliderGetRect(hDlg,nIdParent,&rcParent);

  width=MIN_WIDTH;
  width+=q*(rcParent.right-rcParent.left-MIN_WIDTH)+0.5;
  rcSlider.right=rcSlider.left+width;

  // resize the slider.
  MoveWindow(
    GetDlgItem(hDlg,nIdSlider),
                                // _In_  HWND hWnd,
    rcSlider.left,              // _In_  int X,
    rcSlider.top,               // _In_  int Y,
    width,                      // _In_  int nWidth,
    rcSlider.bottom-rcSlider.top,
                                // _In_  int nHeight,
    TRUE                        // _In_  BOOL bRepaint
  );

  // set the maximum to the parent's position.
  SendMessage(
    GetDlgItem(hDlg,nIdSlider),
                                // _In_  HWND hWnd,
    TBM_SETRANGE,               // _In_  UINT Msg,
    FALSE,                      // _In_  WPARAM wParam (redraw flag),
    MAKELONG(CUST_SLIDER_MIN,pos)
                                // _In_  LPARAM lParam
  );
}

void CustSliderSetText(HWND hDlg, int nIdStatic, int nIdSlider)
{
  double scale=CustSliderGet(hDlg,nIdSlider);
  char buf[128];

  sprintf(buf,"%.2f \xD7 minimum",scale);

  SetWindowTextA(
    GetDlgItem(hDlg,nIdStatic), // _In_      HWND hWnd,
    buf                         // _In_opt_  LPCTSTR lpString
  );
}

///////////////////////////////////////////////////////////////////////////////
static void CustSliderTypeInit(const Control *pControl, HWND hDlg)
{
}

static void CustSliderTypeSet(const Control *pControl, HWND hDlg,
    const void *pData)
{
  const ControlCustSliderConfig *config=pControl->config;

  if (0<config->idcSliderParent)
    CustSliderSetWidth(hDlg,config->idcSlider,config->idcSliderParent);

  SendMessageW(
    GetDlgItem(hDlg,config->idcSlider),
                                // _In_  HWND hWnd,
    TBM_SETRANGE,               // _In_  UINT Msg,
    FALSE,                      // _In_  WPARAM wParam (redraw flag),
    MAKELONG(CUST_SLIDER_MIN,CUST_SLIDER_MAX)
                                // _In_  LPARAM lParam
  );

  SendMessageW(
    GetDlgItem(hDlg,config->idcSlider),
                                // _In_  HWND hWnd,
    TBM_SETPOS,                 // _In_  UINT Msg,
    TRUE,                       // _In_  WPARAM wParam (redraw flag),
    VALUE_DOUBLE(pData,pControl->offset)*CUST_SLIDER_SCALE+0.5
                                // _In_  LPARAM lParam
  );

  CustSliderSetText(hDlg,config->idcStatic,config->idcSlider);
}

static void CustSliderTypeGet(const Control *pControl, HWND hDlg,
    void *pData)
{
  const ControlCustSliderConfig *config=pControl->config;

  VALUE_DOUBLE(pData,pControl->offset)=SendMessageW(
    GetDlgItem(hDlg,config->idcSlider),
                                // _In_  HWND hWnd,
    TBM_GETPOS,                 // _In_  UINT Msg,
    0,                          // _In_  WPARAM wParam,
    0                           // _In_  LPARAM lParam
  );

  VALUE_DOUBLE(pData,pControl->offset)/=CUST_SLIDER_SCALE;
}

const ControlType gcCustSliderType={
  CustSliderTypeInit,
  CustSliderTypeSet,
  CustSliderTypeGet
};
